from contextlib import suppress
from pathlib import Path
from typing import NamedTuple, Optional

import yaml

home = Path.home()
secrets = home / ".secrets.yml"


class Config(NamedTuple):
    phone: str


def configure(phone: str) -> int:
    conf = Config(phone=phone)
    existing = __read_config()

    entry = __get_onefa_entry(existing)

    # create an empty entry if no existing entry
    if entry is None:
        entry = {"app": "onefa", "env": []}
        existing.append(entry)

    # turn conf into a list of { "name": string, "value": string }
    entry["env"] = [{"name": k, "value": v} for k, v in conf._asdict().items()]

    __write_config(existing)

    return 0


def dump_config() -> int:
    conf = get_config()

    if conf is None:
        print("No configuration yet :(")
        return 1

    print("Configuration")
    for key, value in conf._asdict().items():
        print(key, value)

    return 0


def get_config() -> Optional[Config]:
    all_config = __read_config()

    conf = __get_onefa_entry(all_config)
    if conf is None:
        return

    with suppress(Exception):
        return Config(**{var["name"]: var["value"] for var in conf["env"]})


def __read_config():
    with suppress(FileNotFoundError):
        with open(secrets, "r") as fin:
            return yaml.safe_load(fin)

    return []


def __write_config(config):
    with open(secrets, "w+") as fout:
        yaml.dump(config, fout)


def __get_onefa_entry(config):
    for conf in config:
        if conf.get("app") == "onefa":
            return conf



import re
import sys
import time
from contextlib import suppress
from typing import NamedTuple, Optional

import clipboard as clip
import requests

from onefactorauth.config import get_config


class Passcode(NamedTuple):
    code: str
    time: int


class ParseConf(NamedTuple):
    pattern: re.Pattern
    timeout: int
    max_time: int


def dump_passcode(pconf: ParseConf, clipboard: bool) -> int:
    passcode = get_passcode(pconf)
    if passcode is None:
        print("Err: no passcode found", file=sys.stderr)
        return 1

    print(passcode)
    if clipboard:
        clip.copy(passcode)

    return 0


def get_passcode(pconf: ParseConf) -> Optional[str]:
    start = time.time()
    conf = get_config()

    if conf is None:
        return

    s = requests.Session()

    # it needs to first set some cookies before we get latest sms
    s.get(__get_sms_url(conf.phone))

    while time.time() - start <= pconf.timeout:
        r = s.get(__get_sms_url(conf.phone))
        code = __parse_passcode(r.text, pconf.pattern)
        if code is None:
            continue
        if code.time > pconf.max_time:
            return
        return code.code


def __parse_passcode(html: str, passcode_pattern: re.Pattern) -> Optional[Passcode]:
    passcodes = re.findall(passcode_pattern, html)
    if not passcodes:
        return
    timestamp = __parse_passcode_timestamp(html, passcodes[0])
    if not timestamp:
        return
    return Passcode(code=passcodes[0], time=timestamp)


def __parse_passcode_timestamp(html: str, code) -> Optional[int]:
    code_idx = html.find(code)
    date_start = html.rfind("<date>", 0, code_idx)
    date_end = html.rfind("</date>", 0, code_idx)
    if -1 in (date_start, date_end):
        return

    # <date> describes when sms was sent and can be in one of three formats
    # 1. (x minutes ago)
    # 2. (x seconds ago)
    # 3. () - means just sent
    date = html[date_start:date_end]
    minute_match = re.search(r"(\d+) minutes ago", date)
    if minute_match:
        return int(minute_match.group(1))

    # we return in minutes so no need to calculate exact time
    # for seconds and instant
    if "seconds ago" in date or "()" in date:
        return 0


def __get_sms_url(phone: str) -> str:
    return f"https://receivesms.cc/sms/{phone}"



import re
import sys
from argparse import ArgumentParser

from onefactorauth.code import ParseConf, dump_passcode
from onefactorauth.config import configure, dump_config


def main():
    parser = ArgumentParser(description="One factor authentication.")
    commands = parser.add_subparsers(dest="command", help="subcommand")

    config_parser = commands.add_parser("config", description="Configure 1fa")
    config_parser.add_argument(
        "-p",
        "--phone",
        help="phone number to use from https://receivesms.cc/sms/",
        type=str,
    )

    code_parser = commands.add_parser("code", description="Get 1fa code")
    code_parser.add_argument(
        "-c", "--clipboard", help="copy to clipboard", action="store_true"
    )
    code_parser.add_argument(
        "-t", "--timeout", help="timeout (s)", type=int, default=10
    )
    code_parser.add_argument(
        "-p",
        "--pattern",
        help="regex pattern for passcode",
        type=re.compile,
        default=re.compile("SMS p\*\*\*codes: (\d+)"),  # pattern for UCLA duo
    )
    code_parser.add_argument(
        "-m",
        "--max-time",
        help="maximum age of the sms code msg (min)",
        type=int,
        default=2,
    )

    args = parser.parse_args()

    if args.command == "config":
        if args.phone is None:
            return dump_config()
        return configure(args.phone)

    if args.command == "code":
        return dump_passcode(
            ParseConf(
                **{k: v for k, v in args.__dict__.items() if k in ParseConf._fields}
            ),
            args.clipboard,
        )

    print(
        "Err: Please provide a subcommand to run, run --help for more info",
        file=sys.stderr,
    )
    return 1



import sys

import setuptools

# Thank you @KentoNishi for this setup.py

try:
    with open("README.md", "r") as fh:
        long_description = fh.read()
except:
    long_description = "A tool to bypass 2 factor authentication."

try:
    with open("requirements.txt", "r") as reqs:
        requirements = reqs.read().split("\n")
except:
    requirements = ""

try:
    if "refs/tags/v" in sys.argv[1]:
        versionName = sys.argv[1].replace("refs/tags/v", "")
        del sys.argv[1]
    else:
        raise Exception
except:
    versionName = "0.1.0"

setuptools.setup(
    name="onefactorauth",
    version=versionName,
    author="Ronak Badhe",
    author_email="ronak.badhe@gmail.com",
    description=long_description.split("\n")[1],
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/r2dev2/onefactorauth",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Public Domain",
        "Operating System :: OS Independent",
    ],
    entry_points={
        "console_scripts": [
            "onefactorauth=onefactorauth:main",
            "1fa=onefactorauth:main",
        ]
    },
    python_requires=">=3.5",
    install_requires=requirements,
)
